<?php

namespace Drupal\element_pack\Element;

use Drupal\Core\Render\Attribute\RenderElement;
use Drupal\Core\Render\Element\RenderElementBase;
use Drupal\Core\Render\Markup;

/**
 * Provides a custom element for creating a "Go Back" button.
 *
 * This custom render element generates a button-like component that allows 
 * users to navigate back in their browser history. The element is flexible 
 * and can be customized to suit various use cases.
 *
 * ### Key Features:
 * - **Browser History Navigation**: Implements a simple "Go Back" functionality using the browser's history API.
 * - **Customizable Label**: The button label can be customized via the `#label` property.
 * - **Lightweight Markup**: Generates a `<button>` tag with semantic HTML for easy styling and integration.
 *
 * ### Example Usage:
 * The following demonstrates how to use the `history_back_button` element in a render array:
 *
 * @code
 * $build['go_back_button'] = [
 *   '#type' => 'history_back_button',
 *   '#label' => 'Go Back to Dashboard',
 * ];
 * @endcode
 *
 * ### Default Properties:
 * - `#label`: The text displayed on the button. Defaults to "Go Back".
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/API/History/back
 */
#[RenderElement('history_back_button')]
class HistoryBackButton extends RenderElementBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    return [
      '#pre_render' => [
        [self::class, 'preRenderElement'],
      ],
      '#label' => 'Go Back',
    ];
  }

  /**
   * History back button element pre render callback.
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   *
   * @return array
   *   The modified element.
   */
  public static function preRenderElement(array $element): array {
    $label = $element['#label'] ?? 'Go Back';
    $element['#markup'] = Markup::create(sprintf(
      '<button type="button" onclick="history.back()">%s</button>',
      $label
    ));
    return $element;
  }

}
